@@ -1,4 +1,4 @@
-/* $Id: defines.h,v 1.17 2004/10/26 12:45:54 dtucker Exp $ */
+/* $Id: defines.h,v 1.23 2005/05/23 00:16:15 dtucker Exp $ */
 
 /*
  * Copyright (c) 2004 Darren Tucker.
@@ -24,7 +24,9 @@
 # if defined(HAVE_STRUCT_SOCKADDR_SA_LEN)
 #  define SA_LEN(x)	((x)->sa_len)
 # else
-#  define SA_LEN(x)	(sizeof(struct sockaddr))
+#  define SA_LEN(x)     ((x)->sa_family == AF_INET6 ? \
+			sizeof(struct sockaddr_in6) : \
+			sizeof(struct sockaddr_in))
 # endif
 #endif
 
@@ -50,13 +52,6 @@
 # undef MODEMASK
 #endif
 
-#if 0
-/* FreeBSD 5.x does not have EAI_NODATA */
-#ifndef EAI_NODATA
-# define EAI_NODATA EAI_NONAME
-#endif
-#endif
-
 #ifndef MAX
 # define MAX(a,b) (((a)>(b))?(a):(b))
 #endif
@@ -69,6 +64,12 @@
 # define __attribute__(x)
 #endif /* !defined(__GNUC__) || (__GNUC__ < 2) */
 
+#if !defined(HAVE___func__) && defined(HAVE___FUNCTION__)
+#  define __func__ __FUNCTION__
+#elif !defined(HAVE___func__)
+#  define __func__ ""
+#endif
+
 /* Types */
 
 /* If sys/types.h does not supply intXX_t, supply them ourselves */
@@ -168,7 +169,11 @@ typedef unsigned long long int u_int64_t;
 #endif
 
 #ifndef HAVE_SOCKLEN_T
+# ifdef SOCKLEN_T_IS_SIZE_T
+typedef size_t socklen_t;
+# else
 typedef int socklen_t;
+# endif
 #endif
 
 /* FSF bison 1.875 wants this to prevent conflicting definitions of YYSTYPE */
@@ -189,3 +194,17 @@ typedef int socklen_t;
     defined(HAVE_STRUCT_SOCKADDR_STORAGE___SS_FAMILY)
 # define ss_family __ss_family
 #endif
+
+#if defined(BROKEN_GETADDRINFO) && defined(HAVE_GETADDRINFO)
+# undef HAVE_GETADDRINFO
+#endif
+#if defined(BROKEN_GETADDRINFO) && defined(HAVE_FREEADDRINFO)
+# undef HAVE_FREEADDRINFO
+#endif
+#if defined(BROKEN_GETADDRINFO) && defined(HAVE_GAI_STRERROR)
+# undef HAVE_GAI_STRERROR
+#endif
+
+#ifndef __dead
+# define __dead
+#endif
