@@ -18,7 +18,7 @@
 #define OPENNTPD_INCLUDES_H
 
 #define RCSID(msg) \
-static /**/const char *const rcsid[] = { (char *)rcsid, "\100(#)" msg }
+static /**/const char *const rcsid[] = { (const char *)rcsid, "\100(#)" msg }
 
 #include "config.h"
 #include "version.h"
@@ -33,6 +33,7 @@ static /**/const char *const rcsid[] = { (char *)rcsid, "\100(#)" msg }
 #include <limits.h>
 #include <string.h>
 #include <fcntl.h>
+#include <errno.h>
 
 #ifdef HAVE_CTYPE_H
 # include <ctype.h>
@@ -46,6 +47,9 @@ static /**/const char *const rcsid[] = { (char *)rcsid, "\100(#)" msg }
 #ifdef HAVE_SYS_TIME_H
 # include <sys/time.h>
 #endif
+#ifdef HAVE_SYS_TIMERS_H
+# include <sys/timers.h>
+#endif
 #ifdef HAVE_STDINT_H
 # include <stdint.h>
 #endif
@@ -55,6 +59,9 @@ static /**/const char *const rcsid[] = { (char *)rcsid, "\100(#)" msg }
 #ifdef HAVE_SYS_STAT_H
 # include <sys/stat.h>
 #endif
+#ifdef HAVE_SYS_SOCKET_H
+# include <sys/socket.h>
+#endif
 #ifdef HAVE_SYS_FCNTL_H
 # include <sys/fcntl.h>
 #endif
@@ -70,6 +77,14 @@ static /**/const char *const rcsid[] = { (char *)rcsid, "\100(#)" msg }
 #ifdef HAVE_SYSLOG_H
 # include <syslog.h>
 #endif
+#ifdef HAVE_ARPA_NAMESER_H
+# include <arpa/nameser.h>
+#endif
+
+/* on IRIX, sa_len is a macro sys/socket.h that causes conflicts */
+#ifdef sa_len
+# undef sa_len
+#endif
 
 #include "defines.h"
 #ifndef HAVE_SYS_QUEUE_H
