@@ -1,4 +1,4 @@
-/*	$OpenBSD: ntpd.c,v 1.25 2004/12/06 20:57:17 mickey Exp $ */
+/*	$OpenBSD: ntpd.c,v 1.32 2005/03/13 10:06:27 dtucker Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -18,8 +18,7 @@
 
 #include "includes.h"
 
-static const char version_string[] =
-    "@(#)$Release: OpenNTPD "OPENNTPD_VERSION" $";
+RCSID("$Release: OpenNTPD "OPENNTPD_VERSION" $");
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -28,8 +27,11 @@ static const char version_string[] =
 #include <arpa/inet.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <poll.h>
+#ifdef HAVE_POLL_H
+# include <poll.h>
+#endif
 #include <pwd.h>
+#include <resolv.h>
 #include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -38,13 +40,13 @@ static const char version_string[] =
 
 #include "ntpd.h"
 
-void	sighdlr(int);
-void	usage(void);
-int	main(int, char *[]);
-int	check_child(pid_t, const char *);
-int	dispatch_imsg(struct ntpd_conf *);
-void	ntpd_adjtime(double);
-void	ntpd_settime(double);
+void		sighdlr(int);
+__dead void	usage(void);
+int		main(int, char *[]);
+int		check_child(pid_t, const char *);
+int		dispatch_imsg(struct ntpd_conf *);
+void		ntpd_adjtime(double);
+void		ntpd_settime(double);
 
 volatile sig_atomic_t	 quit = 0;
 volatile sig_atomic_t	 reconfig = 0;
@@ -68,7 +70,7 @@ sighdlr(int sig)
 	}
 }
 
-void
+__dead void
 usage(void)
 {
 	extern char *__progname;
@@ -98,6 +100,7 @@ main(int argc, char *argv[])
 	bzero(&conf, sizeof(conf));
 
 	log_init(1);		/* log to stderr until daemonized */
+	res_init();		/* XXX */
 
 	while ((ch = getopt(argc, argv, "df:sS")) != -1) {
 		switch (ch) {
@@ -140,14 +143,15 @@ main(int argc, char *argv[])
 	if (!conf.settime) {
 		log_init(conf.debug);
 		if (!conf.debug)
-			daemon(1, 0);
+			if (daemon(1, 0))
+				fatal("daemon");
 	} else
 		timeout = 15 * 1000;
 
 	if (socketpair(AF_UNIX, SOCK_STREAM, PF_UNSPEC, pipe_chld) == -1)
 		fatal("socketpair");
 
-	/* fork children */
+	/* fork child process */
 	chld_pid = ntp_main(pipe_chld, &conf);
 
 	setproctitle("[priv]");
@@ -176,18 +180,19 @@ main(int argc, char *argv[])
 			}
 
 		if (nfds == 0 && conf.settime) {
-			log_debug("no reply received, skipping initial time"
-			    "setting");
 			conf.settime = 0;
 			timeout = INFTIM;
 			log_init(conf.debug);
+			log_debug("no reply received, skipping initial time "
+			    "setting");
 			if (!conf.debug)
-				daemon(1, 0);
+				if (daemon(1, 0))
+					fatal("daemon");
 		}
 
 		if (nfds > 0 && (pfd[PFD_PIPE].revents & POLLOUT))
 			if (msgbuf_write(&ibuf->w) < 0) {
-				log_warn("pipe write error (to child");
+				log_warn("pipe write error (to child)");
 				quit = 1;
 			}
 
@@ -286,7 +291,8 @@ dispatch_imsg(struct ntpd_conf *conf)
 			/* daemonize now */
 			log_init(conf->debug);
 			if (!conf->debug)
-				daemon(1, 0);
+				if (daemon(1, 0))
+					fatal("daemon");
 			conf->settime = 0;
 			break;
 		case IMSG_HOST_DNS:
@@ -318,7 +324,8 @@ ntpd_adjtime(double d)
 {
 	struct timeval	tv;
 
-	if (d >= (double)LOG_NEGLIGEE / 1000)
+	if (d >= (double)LOG_NEGLIGEE / 1000 ||
+	    d <= -1 * (double)LOG_NEGLIGEE / 1000)
 		log_info("adjusting local clock by %fs", d);
 	else
 		log_debug("adjusting local clock by %fs", d);
@@ -338,17 +345,19 @@ ntpd_settime(double d)
 	if (d < SETTIME_MIN_OFFSET && d > -SETTIME_MIN_OFFSET)
 		return;
 
-	d_to_tv(d, &tv);
-	if (gettimeofday(&curtime, NULL) == -1)
+	if (gettimeofday(&curtime, NULL) == -1) {
 		log_warn("gettimeofday");
-	curtime.tv_sec += tv.tv_sec;
-	curtime.tv_usec += tv.tv_usec;
-	if (curtime.tv_usec > 1000000) {
-		curtime.tv_sec++;
-		curtime.tv_usec -= 1000000;
+		return;
 	}
-	if (settimeofday(&curtime, NULL) == -1)
+	d_to_tv(d, &tv);
+	curtime.tv_usec += tv.tv_usec + 1000000;
+	curtime.tv_sec += tv.tv_sec - 1 + (curtime.tv_usec / 1000000);
+	curtime.tv_usec %= 1000000;
+
+	if (settimeofday(&curtime, NULL) == -1) {
 		log_warn("settimeofday");
+		return;
+	}
 	tval = curtime.tv_sec;
 	strftime(buf, sizeof(buf), "%a %b %e %H:%M:%S %Z %Y",
 	    localtime(&tval));
