@@ -1,4 +1,4 @@
-/*	$OpenBSD: ntpd.h,v 1.46 2004/12/06 20:57:17 mickey Exp $ */
+/*	$OpenBSD: ntpd.h,v 1.53 2005/02/03 10:53:33 dtucker Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -47,6 +47,9 @@
 #define	TRUSTLEVEL_BADPEER		6
 #define	TRUSTLEVEL_PATHETIC		2
 #define	TRUSTLEVEL_AGRESSIVE		8
+#define	TRUSTLEVEL_MAX			10
+
+#define	MAX_SERVERS_DNS			8
 
 #define	QSCALE_OFF_MIN			0.05
 #define	QSCALE_OFF_MAX			0.50
@@ -112,6 +115,7 @@ struct ntp_peer {
 	u_int32_t			 id;
 	u_int8_t			 shift;
 	u_int8_t			 trustlevel;
+	int				 lasterror;
 };
 
 struct ntpd_conf {
@@ -121,14 +125,15 @@ struct ntpd_conf {
 	u_int8_t				listen_all;
 	u_int8_t				settime;
 	u_int8_t				debug;
+	u_int32_t				scale;
 };
 
 struct buf {
 	TAILQ_ENTRY(buf)	 entries;
 	u_char			*buf;
-	ssize_t			 size;
-	ssize_t			 wpos;
-	ssize_t			 rpos;
+	size_t			 size;
+	size_t			 wpos;
+	size_t			 rpos;
 };
 
 struct msgbuf {
@@ -187,8 +192,8 @@ void		 fatalx(const char *);
 const char *	 log_sockaddr(struct sockaddr *);
 
 /* buffer.c */
-struct buf	*buf_open(ssize_t);
-int		 buf_add(struct buf *, void *, ssize_t);
+struct buf	*buf_open(size_t);
+int		 buf_add(struct buf *, void *, size_t);
 int		 buf_close(struct msgbuf *, struct buf *);
 void		 buf_free(struct buf *);
 void		 msgbuf_init(struct msgbuf *);
@@ -236,6 +241,11 @@ int	client_addr_init(struct ntp_peer *);
 int	client_nextaddr(struct ntp_peer *);
 int	client_query(struct ntp_peer *);
 int	client_dispatch(struct ntp_peer *, u_int8_t);
+void	client_log_error(struct ntp_peer *, const char *, int);
+void	update_scale(double);
+time_t	scale_interval(time_t);
+time_t	error_interval(void);
+void	set_next(struct ntp_peer *, time_t);
 
 /* util.c */
 double			gettime(void);
