@@ -1,4 +1,4 @@
-/* $Id: openbsd-compat.h,v 1.12 2004/12/03 02:17:50 dtucker Exp $ */
+/* $Id: openbsd-compat.h,v 1.15 2004/12/19 03:04:22 dtucker Exp $ */
 
 /*
  * Copyright (c) 2004 Darren Tucker.
@@ -20,8 +20,10 @@
 #include <netdb.h>
 
 #include "fake-rfc2553.h"
+#include "atomicio.h"
+#include "bsd-poll.h"
 
-char *_compat_get_progname(const char *);
+char *_compat_get_progname(char *);
 
 #ifndef HAVE_ARC4RANDOM
 void seed_rng(void);
