@@ -1,4 +1,4 @@
-/*	$OpenBSD: server.c,v 1.18 2004/10/22 21:17:37 henning Exp $ */
+/*	$OpenBSD: server.c,v 1.21 2005/01/28 12:01:32 dtucker Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -41,13 +41,13 @@ setup_listeners(struct servent *se, struct ntpd_conf *conf, u_int *cnt)
 
 	if (conf->listen_all) {
 #ifdef HAVE_GETIFADDRS
-		struct ifaddrs		*ifap;
+		struct ifaddrs		*ifa, *ifap;
 		struct sockaddr		*sa;
 
-		if (getifaddrs(&ifap) == -1)
+		if (getifaddrs(&ifa) == -1)
 			fatal("getifaddrs");
 
-		for (; ifap != NULL; ifap = ifap->ifa_next) {
+		for (ifap = ifa; ifap != NULL; ifap = ifap->ifa_next) {
 			sa = ifap->ifa_addr;
 
 			if (sa->sa_family != AF_INET &&
@@ -62,7 +62,7 @@ setup_listeners(struct servent *se, struct ntpd_conf *conf, u_int *cnt)
 			TAILQ_INSERT_TAIL(&conf->listen_addrs, la, entry);
 		}
 
-		freeifaddrs(ifap);
+		freeifaddrs(ifa);
 #else
 		fprintf(stderr, "\"listen on *\" not supported on this "
 		    "platform, interface address required\n");
@@ -123,7 +123,7 @@ server_dispatch(int fd, struct ntpd_conf *conf)
 	if ((size = recvfrom(fd, &buf, sizeof(buf), 0,
 	    (struct sockaddr *)&fsa, &fsa_len)) == -1) {
 		if (errno == EHOSTUNREACH || errno == EHOSTDOWN ||
-		    errno == ENETDOWN) {
+		    errno == ENETUNREACH || errno == ENETDOWN) {
 			log_warn("recvfrom %s",
 			    log_sockaddr((struct sockaddr *)&fsa));
 			return (0);
@@ -155,7 +155,7 @@ server_dispatch(int fd, struct ntpd_conf *conf)
 	reply.rootdelay = d_to_sfp(conf->status.rootdelay);
 
 	if (version > 3)
-		reply.refid = reply.xmttime.fraction;
+		reply.refid = reply.xmttime.fractionl;
 	else
 		reply.refid = conf->status.refid;
 
